;################################################################## 
;#  BBSOFITS Copyright: California Institute of Technology, 1996  # 
;#       (BBSOFITS is freeware and cannot be sold)                # 
;################################################################## 
;#    B B S O F I T S  -  IDL widget for image display and        # 
;#    processing. Can read FITS and JPEG but can output also      # 
;#    GIF and POSTSCRIPT.                                         # 
;#                                    ver 3.0                     # 
;#                                                                # 
;# 2.1             Written by: Anders Johannesson & Jamey Bass    # 
;#                             Big Bear Solar Observatory         # 
;#                             Summer, 1996                       # 
;# 2.1.1          Small modifications by Scot Kleinman Oct96      # 
;#                  This version adds rebinning to edit menu      # 
;# 2.2            Additional Mods by SJK Nov96 - Now allows       # 
;#                  Dark and Flat Field Corrections.              # 
;#                  Records operation on FITs files in the        # 
;#                  headers with History lines and appropriate    # 
;#                  changes of NAXIS1 and NAXIS2 and records this # 
;#                  proper header when saving a FITS file         # 
;# 3.0            Modifications Nov. 96 by Anders Johannesson and # 
;#                Scot Kleinman. New features:                    # 
;#                  Annotation                                    # 
;#                  Log, Power, Improved Histogram, Laplace filt. # 
;#                  Arbitrary Rotation Angle, Easier installation # 
;#                  + various bug fixes                           # 
;#                                                                # 
;################################################################## 
 
;# FILL SELECTION ARRAY ##################### 
function fill_select,status,inver,p1,p2,first 
 
;# CREATE ARRAY 
out=fltarr(14) 
 
;# SET STATUS AN INVERTATION FIELDS 
out(0)=status 
out(1)=inver 
 
;# SET X-COORDINATE FIELDS 
out(2)=p1(0)<p2(0) 
out(3)=out(2) 
out(4)=p1(0)>p2(0) 
out(5)=out(4) 
out(6)=out(2) 
 
;# SET Y-COORDINATE FIELDS 
out(7)=p1(1)<p2(1) 
out(8)=p1(1)>p2(1) 
out(9)=out(8) 
out(10)=out(7) 
out(11)=out(7) 
 
;# SET FIRST CORNER 
out(12)=first(0) 
out(13)=first(1) 
 
return,out 
 
end 
 
;# SHOW CURRENT SELECTION ########################################### 
pro show_select 
 
common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
       outname,helpfile,selection,oselection,traceplot,saveflag 
 
;# REDRAW IMAGE 
tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
 
;# INNER SELECTION 
if (selection(2) eq selection(4)) and (selection(7) eq selection(8)) then begin 
   plots,selection(2:6),selection(7:11),psym=1,thick=2,/device 
endif else begin 
   plots,selection(2:6),selection(7:11),linestyle=2,thick=2,/device 
endelse 
 
;# OUTER BORDER IF INVERT IS SELECTED 
if selection(1) eq 1 then begin 
   x1=0.0 
   x2=dim(1)-1.0 
   y1=0.0+(imsize(1)-dim(2)) 
   y2=(dim(2)-1)+(imsize(1)-dim(2)) 
   plots,[x1,x1,x2,x2,x1],[y1,y2,y2,y1,y1],linestyle=2,thick=2,/device 
endif 
 
end 
 
 
;# FILE MENU ########### 
PRO PDMENU3_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname, helpfile, selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
 
  CASE Event.Value OF 
 
  'File.Open': BEGIN 
    bbsoread,' Select FITS or JPEG Image' 
   END 
  'File.Save': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    saveflag=1 
    if strlen(outname) gt 1 then begin 
       WIDGET_CONTROL, /HOURGLASS 
       if (min(im) ge 0) and (max(im) lt 256) then begin 
          writefits,outname,byte(im),header 
       endif else begin 
          writefits,outname,im,header 
       endelse 
    endif else begin 
       WIDGET_CONTROL, /HOURGLASS 
       outname=pickfile(GET_PATH=newpath,FILTER='*.fts') 
       if strlen(outname) gt 1 then begin 
          ;cd,strmid(outname,0,rstrpos(outname,'\')) 
          cd,newpath 
          WIDGET_CONTROL, /HOURGLASS 
          if (min(im) ge 0) and (max(im) lt 256) then begin 
             writefits,outname,byte(im),header 
          endif else begin 
             writefits,outname,im,header 
          endelse 
       endif 
    endelse 
   END 
  'File.Save as': BEGIN 
    saveflag=1 
    WIDGET_CONTROL, /HOURGLASS 
    outname=pickfile(GET_PATH=newpath,FILTER='*.fts') 
    if strlen(outname) gt 1 then begin 
       ;cd,strmid(outname,0,rstrpos(outname,'\')) 
       cd,newpath 
       WIDGET_CONTROL, /HOURGLASS 
       if (min(im) ge 0) and (max(im) lt 256) then begin 
          writefits,outname,byte(im),header 
       endif else begin 
          writefits,outname,im,header 
       endelse 
    endif 
    END 
  'File.Print': BEGIN 
    ;# GET FILE NAME 
    WIDGET_CONTROL, /HOURGLASS 
    printfile=pickfile(FILTER='*.ps') 
    WIDGET_CONTROL, /HOURGLASS 
    if strlen(printfile) gt 1 then begin 
       bbsoprin,dim,imsize,printfile,header,filename 
    endif 
    END 
  'File.to gif': BEGIN 
    saveflag=1 
    WIDGET_CONTROL, /HOURGLASS 
    gifname=pickfile(FILTER='*.gif') 
    if strlen(gifname) gt 1 then begin 
       WIDGET_CONTROL, /HOURGLASS 
       out=bytscl(im,contrast(0),contrast(1)) 
       tvlct,r,g,b,/get 
       write_gif,gifname,out,r,g,b 
    endif 
    END 
  'File.to jpeg': BEGIN 
    saveflag=1 
    WIDGET_CONTROL, /HOURGLASS 
    jpgname=pickfile(FILTER='*.jpg') 
    if strlen(jpgname) gt 1 then begin 
       WIDGET_CONTROL, /HOURGLASS 
       out=bytscl(im,contrast(0),contrast(1)) 
       out=byte(round(out*255.0/float(max(out)))>0<255) 
       write_jpeg,jpgname,out 
    endif 
    END 
  'File.Restore': BEGIN 
    saveflag=1 
    if strlen(filename) gt 1 then begin 
       WIDGET_CONTROL, /HOURGLASS 
       ;cd,strmid(filename,0,rstrpos(filename,'\')) 
       extension=strmid(filename,rstrpos(filename,'.')+1,3) 
       if (extension eq 'fts') or (extension eq 'fit') then begin 
          im=readfits(filename,header,/SILENT,/NOSCALE) 
          imlast=im 
          dim=size(im) 
          ;# RESET SELECTION 
          p1=[0.0,0.0+(imsize(1)-dim(2))] 
          p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
          selection=fill_select(0.0,0.0,p1,p2,p1) 
          oselection=selection 
          ;# MAKE BYTE SHORTWORD 
          if dim(3) eq 1 then im=fix(im) 
          pim=im(40:dim(1)-41,40:dim(2)-41) 
          contrast=[long(min(pim)),long(max(pim))] 
          ocontrast=contrast 
          erase 
          tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
          loadct,0 
          WIDGET_CONTROL, IMAGETEXT, set_value=' '+$ 
          strmid(filename,rstrpos(filename,'\')+1,99) 
       endif 
       if (extension eq 'jpg') then begin 
          read_jpeg,filename,im 
          header='NO HEADER - image was read as JPEG' 
          imlast=im 
          dim=size(im) 
          ;# RESET SELECTION 
          p1=[0.0,0.0+(imsize(1)-dim(2))] 
          p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
          selection=fill_select(0.0,0.0,p1,p2,p1) 
          oselection=selection 
          ;# MAKE BYTE SHORTWORD 
          if dim(3) eq 1 then im=fix(im) 
          pim=im(40:dim(1)-41,40:dim(2)-41) 
          contrast=[long(min(pim)),long(max(pim))] 
          ocontrast=contrast 
          erase 
          tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
          loadct,0 
          WIDGET_CONTROL, IMAGETEXT, set_value=' '+$ 
          strmid(filename,rstrpos(filename,'\')+1,99) 
       endif 
    endif else begin 
       WIDGET_CONTROL, IMAGETEXT, set_value=' No image loaded' 
    endelse 
    END 
  'File.to IDL': BEGIN 
    if saveflag eq 0 then begin 
       ierr=execute('bbsoidl,GROUP=event.top') 
    endif else begin 
       WIDGET_CONTROL,event.top,/destroy 
    endelse 
    END 
  'File.Quit': BEGIN 
    if saveflag eq 0 then begin 
       ierr=execute('bbsoexit,GROUP=event.top') 
    endif else begin 
       exit 
    endelse 
    END 
  ENDCASE 
END 
 
 
; DO NOT REMOVE THIS COMMENT: END PDMENU3 
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST. 
 
 
; CODE MODIFICATIONS MADE ABOVE THIS COMMENT WILL BE LOST. 
; DO NOT REMOVE THIS COMMENT: BEGIN PDMENU23 
 
 
;# EDIT MENU # 
PRO PDMENU23_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
 
  ;# DISABLE 'IMAGE SAVED' FLAG 
  saveflag=0 
 
  CASE Event.Value OF 
 
  'Edit.Correct.Subtract': BEGIN 
    bbsosubt,'Select Image to Subtract',0 
    END 
  'Edit.Correct.Dark': BEGIN 
    bbsosubt,'Select Dark Image',1 
    END 
  'Edit.Correct.Flat': BEGIN 
    bbsoflat 
    END 
  'Edit.Display.Contrast.Bytescl': BEGIN 
    ierr=execute('bbsobyts,GROUP=event.top') 
    ;tv,bytscl(im,contrast(0),contrast(1)) 
    END 
  'Edit.Display.Contrast.LUT': BEGIN 
    ierr=execute('xloadct,/SILENT,GROUP=event.top') 
    END 
  'Edit.Display.Color Table.BW ramp': BEGIN 
    loadct,0 
    END 
  'Edit.Display.Color Table.Blue ramp': BEGIN 
    loadct,1 
    END 
  'Edit.Display.Color Table.Orange ramp': BEGIN 
    loadct,3 
    END 
  'Edit.Display.Color Table.Red ramp': BEGIN 
    loadct,7 
    END 
  'Edit.Display.Color Table.Green ramp': BEGIN 
    loadct,8 
    END 
  'Edit.Display.Color Table.Other': BEGIN 
    ierr=execute('xloadct,/SILENT,GROUP=event.top') 
    END 
  'Edit.Display.Negative': BEGIN 
    tvlct,r,g,b,/get 
    tvlct,reverse(r),reverse(g),reverse(b) 
    END 
  'Edit.Display.Log': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=fix(round(alog10((float(im)>0.0)+1.0)*100.0)) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/16000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Logarithm',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Display.Log Power': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=abs(fft(im,-1)) 
    im=shift(alog10(im+1.0)*1000.0,dim(1)/2,dim(2)/2) 
    im=fix(round(im)) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/2000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Log Power Spectrum',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Display.Erase': BEGIN 
    erase 
    END 
  'Edit.Filter.High Pass 9x9': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=fix(im)-bbso_bs(fix(im),9,selection,imsize) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/2000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'High Pass Filter 9x9',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Smooth 9x9': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=bbso_bs(im,9,selection,imsize) 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Smooth Filter 9x9',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Sharpen 9x9': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=round(3.0*im-2.0*bbso_bs(im,9,selection,imsize)) 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Sharpen Filter 9x9',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Edge': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=sobel(im) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/2000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Edge Filter',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Laplace': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    kernel=lonarr(3,3)*0L-1L 
    kernel(1,1)=8 
    im=convol(im,kernel,1.0) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/2000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Laplace Filter',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Relief': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=fix(im)-fix(shift(im,1,1)) 
    hist_arr=histogram(im,omin=offset) 
    loc=where(hist_arr gt N_ELEMENTS(im)/2000)+offset 
    contrast=[long(min(loc)),long(max(loc))] 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Relief Filter',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Median 3x3': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=bbsomedi(im,3,selection,imsize) 
    ;Adjust header if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
       oheader=header 
       sxaddhist,'Median Filter 3x3',header 
    endif 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Filter.Custom': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    ierr=execute('bbsofilt,GROUP=event.top') 
    END 
  'Edit.Rebin NxN': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    ierr=execute('bbsorebi,GROUP=event.top') 
    END 
  'Edit.Resize.2 X': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=congrid(im,dim(1)*2.0,dim(2)*2.0,/interp) 
    dim=size(im) 
    ; Adjust the Header  if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Resized 2x',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Resize.0.5 X': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=congrid(im,dim(1)/2.0,dim(2)/2.0,/interp) 
    dim=size(im) 
    ; Adjust the Header  if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Resized 0.5x',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Resize.0.25 X': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=congrid(im,dim(1)/4.0,dim(2)/4.0,/interp) 
    dim=size(im) 
    ; Adjust the Header  if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Resized 0.25x',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Resize.BBSO VMG': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=congrid(im,dim(1),dim(2)*0.8,/interp) 
    dim=size(im) 
    ; Adjust the Header  if it's a FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'BBSO VMG Aspect Ratio correction (0.8)',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.-90': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,1) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated -90',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.180': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,2) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated 180',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.+90': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,3) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated +90',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.hflip': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,5) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated Hflip',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.hflip and -90': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,6) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated Hflip and -90',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.vflip': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,7) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated Vflip',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.hflip and +90': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    im=rotate(im,4) 
    dim=size(im) 
    ;Adjust header if FITS file 
    if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
        oheader=header 
        sxaddpar,header,'NAXIS1',dim(1) 
        sxaddpar,header,'NAXIS2',dim(2) 
        sxaddhist,'Rotated Hflip and +90',header 
    endif 
    ;# RESET SELECTION 
    p1=[0.0,0.0+(imsize(1)-dim(2))] 
    p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
    selection=fill_select(0.0,0.0,p1,p2,p1) 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Rotate.Arbitrary': BEGIN 
    WIDGET_CONTROL, /HOURGLASS 
    ierr=execute('bbso_rot,GROUP=event.top') 
    END 
  'Edit.Annotate': BEGIN 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    WIDGET_CONTROL, /HOURGLASS 
    ierr=execute('Annotate') 
    imb=tvrd(0,imsize(1)-dim(2),dim(1),dim(2)) 
    ima=(bytscl(im,contrast(0),contrast(1)))<max(imb) 
    mask_ab=((ima xor imb) gt 0) 
    graph_level=(mask_ab*imb) 
    loc=where(graph_level eq max(imb)) 
    if n_elements(loc) gt 2 then begin 
       graph_level(loc)=255 
    endif 
    ima=0 & imb=0 
    graph_level=fix(graph_level*(contrast(1)-contrast(0))/255.0+contrast(0)) 
    im=im*(1-mask_ab)+graph_level*mask_ab 
    mask_ab=0 & graph_level=0 
    END 
  'Edit.Byte Swap': BEGIN 
    imlast=im 
    ocontrast=contrast 
    oselection=selection 
    byteorder,im 
    pim=im(40:dim(1)-41,40:dim(2)-41) 
    contrast=[long(min(pim)),long(max(pim))] 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Refresh': BEGIN 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Edit.Undo': BEGIN 
    im=imlast 
    dim=size(im) 
    contrast=ocontrast 
    selection=oselection 
    header=oheader 
    erase 
    tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  ENDCASE 
END 
 
 
; DO NOT REMOVE THIS COMMENT: END PDMENU23 
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST. 
 
 
; CODE MODIFICATIONS MADE ABOVE THIS COMMENT WILL BE LOST. 
; DO NOT REMOVE THIS COMMENT: BEGIN PDMENU7 
 
 
;# INFO MENU # 
PRO PDMENU7_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
  COMMON DRAW26_Comm, DRAW26_Id 
  COMMON DRAW25_Comm, DRAW25_Id 
  COMMON MOUSEH, readpix, marquee, wplot 
 
  CASE Event.Value OF 
 
  'Info.Histogram': BEGIN 
    ;old_window=!D.WINDOW 
    x1=selection(2) 
    x2=selection(5) 
    y1=selection(7)-(imsize(1)-dim(2)) 
    y2=selection(8)-(imsize(1)-dim(2)) 
    show_select 
    ;window,1,xsize=512,ysize=512,title='  Histogram' 
    if selection(1) eq 0 then begin 
       hist_arr=histogram(im(x1:x2,y1:y2),omin=offset) 
    endif else begin 
       hist0=histogram(im,omin=offset0) 
       histp=histogram(im(x1:x2,y1:y2),omin=offsetp) 
       hist_arr=hist0*0 
       hist_arr(offsetp-offset0)=histp 
       hist_arr=hist0-hist_arr 
       offset=offset0 
    endelse 
    if n_elements(hist_arr) eq 1 then begin 
       counts=hist_arr 
       hist_arr=lonarr(41) 
       hist_arr(20)=counts(0) 
       offset=offset-20 
    endif 
    xarr=lindgen(n_elements(hist_arr))+offset 
    if n_elements(hist_arr) gt 41 then begin 
       max_count=max(median(hist_arr,5)) 
    endif else begin 
       max_count=max(hist_arr) 
    endelse 
    ;plot,xarr,hist_arr,xtitle='PIXEL VALUE',yrange=[0,max_count] 
    ;wset,old_window 
    limits=[min(xarr),max(xarr),0,max_count] 
    caption=['HISTOGRAM','Pixel Value','Number of Pixels'] 
    ierr=execute('bbsoplot,xarr,hist_arr,limits,caption,GROUP=event.top') 
    END 
  'Info.Header': BEGIN 
    a=!version & ver_no=fix(strmid(a.release,0,1)) 
    if ver_no gt 3 then begin 
       ShowInfo, TITLE=" FITS header", GROUP=event.top, $ 
                 WIDTH=80, HEIGHT=24, INFOTEXT=header 
    endif else begin 
       old_window=!D.WINDOW 
       window,1,xsize=512,ysize=512,title='  FITS header' 
       for i=0,n_elements(header)-1 do begin 
          xyouts,10,512-(20+i*15),header(i),FONT=0,/device 
       endfor 
       wset,old_window 
    endelse 
    END 
  'Info.Plot': BEGIN 
    wplot=1 
    tv,bytscl(im,contrast(0),contrast(1))>2,0,imsize(1)-dim(2) 
    WIDGET_CONTROL, IMAGETEXT, set_value=' Left click to select first point' 
    END 
  'Info.Pixel Value': BEGIN 
    WIDGET_CONTROL, IMAGETEXT, set_value=' Click:   Left to start    Right to stop' 
    readpix=1 
    END 
  ENDCASE 
END 
 
 
; DO NOT REMOVE THIS COMMENT: END PDMENU7 
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST. 
 
 
; CODE MODIFICATIONS MADE ABOVE THIS COMMENT WILL BE LOST. 
; DO NOT REMOVE THIS COMMENT: BEGIN MAIN13 
 
;# SELECT MENU # 
PRO PDMENU8_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  common MOUSEH, readpix, marquee, wplot 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
 
  CASE Event.Value OF 
 
  'Select.Marquee': BEGIN 
      marquee=1 
      oselection=selection 
      tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
      WIDGET_CONTROL, IMAGETEXT, set_value=' Left click to select first point' 
    END 
  'Select.Invert': BEGIN 
      oselection=selection 
      if selection(1) eq 0 then begin 
         selection(1)=1.0 
      endif else begin 
         selection(1)=0.0 
      endelse 
      show_select 
    END 
  'Select.Crop': BEGIN 
      oselection=selection 
      WIDGET_CONTROL, /HOURGLASS 
      imlast=im 
      ocontrast=contrast 
      x1=selection(2) 
      x2=selection(5) 
      y1=selection(7)-(imsize(1)-dim(2)) 
      y2=selection(8)-(imsize(1)-dim(2)) 
      im=im(x1:x2,y1:y2) 
      dim=size(im) 
      erase 
      tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
      p1=[0.0,0.0+(imsize(1)-dim(2))] 
      p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
      selection=fill_select(0.0,0.0,p1,p2,p1) 
      ; Adjust the Header  if it's a FITS file 
      if (strpos(header(0),'NO HEADER') ne 0) then begin ;Image is FITS 
         oheader=header 
         sxaddpar,header,'NAXIS1',dim(1) 
         sxaddpar,header,'NAXIS2',dim(2) 
         sxaddhist,'Cropped',header 
      endif 
    END 
  'Select.Hide': BEGIN 
      tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
    END 
  'Select.Show': BEGIN 
      tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
      show_select 
    END 
  'Select.Deselect': BEGIN 
      oselection=selection 
      tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
      p1=[0.0,0.0+(imsize(1)-dim(2))] 
      p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
      selection=fill_select(0.0,0.0,p1,p2,p1) 
    END 
 'Select.Save': BEGIN 
      selfile=pickfile(Filter='*.sel',Title=' Give name of selection file') 
      if strlen(selfile) gt 1 then begin 
         save,file=selfile,selection 
      endif 
    END 
 'Select.Restore': BEGIN 
      oselection=selection 
      selfile=pickfile(Filter='*.sel',Title=' Give name of selection file') 
      if strlen(selfile) gt 1 then begin 
         ;# RESTORE FILE 
         restore,selfile 
         ;# IS SELECTION OK? 
         x1=selection(2) 
         y1=dim(2)-(imsize(1)-selection(7)) 
         x2=selection(4) 
         y2=dim(2)-(imsize(1)-selection(8)) 
         if (x1 ge 0) and (x1 lt dim(1)) and (y1 ge 0) and (y1 lt dim(2)) and $ 
            (x2 ge 0) and (x2 lt dim(1)) and (y2 ge 0) and (y2 lt dim(2)) then begin 
            sel_ok=1 
         endif else begin 
            sel_ok=0 
         endelse 
         ;# APPLY IF OK           
         if sel_ok gt 0 then begin 
            tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
            show_select 
         endif else begin 
            selection=oselection 
            ;# MESSAGE 
            a=!version & ver_no=fix(strmid(a.release,0,1)) 
            if ver_no gt 3 then begin 
               r=WIDGET_MESSAGE('Selection outside image'+STRING(13B)+$ 
                             'Old selection remains.') 
            endif else begin 
               mess='Selection outside image - old selection remains' 
               ierr=execute('bbsomess,mess') 
            endelse 
         endelse 
      endif 
    END 
 
  ENDCASE 
END 
 
 
;# HELP MENU # 
PRO PDMENU9_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  CASE Event.Value OF 
 
  'Help.Help': BEGIN 
    openr,fp,helpfile,/GET_LUN 
    helptext='' 
    line='' 
    while not eof(fp) do begin 
       readf,fp,line 
       helptext=[helptext,line] 
    endwhile 
    free_lun,fp 
    a=!version & ver_no=fix(strmid(a.release,0,1)) 
    if ver_no gt 3 then begin 
       ;if strpos(!version.os_family,'Windows') ne -1 then begin 
          ShowInfo, TITLE="  BBSOFITS (ver 3.0) HELP", GROUP=event.top, $ 
                 WIDTH=80, HEIGHT=24, INFOTEXT=helptext 
       ;endif else begin 
       ;   erase 
       ;   text='IDL UNIX/VMS - Please refresh the window by clicking Edit/Refresh when done.' 
       ;   xyouts,10,imsize(1)-20,text,FONT=0,/device 
       ;   for line_no=0,n_elements(helptext)-1 do begin 
       ;      xyouts,10,imsize(1)-40-15*line_no,helptext(line_no),FONT=0,/device 
       ;   endfor 
       ;endelse 
    endif else begin 
       erase 
       text='IDL ver 3.6 - Please refresh the window by clicking Edit/Refresh when done.' 
       xyouts,10,imsize(1)-20,text,FONT=0,/device 
       for line_no=0,n_elements(helptext)-1 do begin 
          xyouts,10,imsize(1)-40-15*line_no,helptext(line_no),FONT=0,/device 
       endfor 
    endelse 
    END 
  'Help.About': BEGIN 
    about_text=["BBSOFITS is an IDL widget tool for basic display and processing "+ $ 
                "of images stored in the FITS and JPEG formats.", + $ 
                "  The code is written in July 1996 by Anders Johannesson & Jamey Bass at the "+$ 
                "Big Bear Solar Observatory.  (c) California Institute of Technology 1996",+$ 
                "Mod. by Anders Johannesson & Scot Kleinman in Nov. 1996."] 
    a=!version & ver_no=fix(strmid(a.release,0,1)) 
    if ver_no gt 3 then begin 
       ShowInfo, TITLE="  About BBSOFITS (ver 3.0)", GROUP=event.top, $ 
                 WIDTH=80, HEIGHT=4, INFOTEXT=about_text 
    endif else begin 
       window,1,xsize=520,ysize=75,title='  About BBSOFITS (ver 3.0)',xpos=200,ypos=200 
       xyouts,10,55,"BBSOFITS is an IDL widget tool for basic display and processing ",$ 
       FONT=0,/device 
       xyouts,10,40,"of images stored in the FITS and JPEG formats.",$ 
       FONT=0,/device 
       xyouts,10,25,"Code by Anders Johannesson & Jamey Bass & Scot Kleinman at the ",$ 
       FONT=0,/device 
       xyouts,10,10,"Big Bear Solar Observatory. (c) California Institute of Technology 1996",$ 
       FONT=0,/device 
    endelse 
    END 
 
  ENDCASE 
END 
 
;# READPIX EVENTS # 
PRO READPIX_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
  COMMON DRAW26_Comm, DRAW26_Id 
  COMMON DRAW25_Comm, DRAW25_Id 
  COMMON MOUSEH, readpix, marquee, wplot 
 
  head_text=' Click:   Left for value   Right to stop' 
 
  ;# DID THE LEFT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press eq 1 then begin 
 
     ;# READ POSITION OF CURSOR AT THIS EVENT 
     x=event.X 
     y=event.Y 
 
     ;# CONVERT TO IMAGE COORDINATES 
     xx=x 
     yy=dim(2)-(imsize(1)-y) 
 
     ;# WRITE SCREEN OUTPUT 
     sx=string(format='(I4.1)',xx) 
     sy=string(format='(I4.1)',yy) 
     if (xx lt dim(1)) and (yy ge 0) then begin 
        svalue=string(float(im(xx,yy))) 
        current_pixel=' '+sx+'  '+sy+'  '+svalue 
        WIDGET_CONTROL, IMAGETEXT, set_value=[head_text,current_pixel] 
        wset,DRAW26_Id 
        level=tvrd(x,y,1,1) 
        wset,DRAW25_Id 
        tv,bytarr(128,40)+level(0) 
        wset,DRAW26_Id 
     endif else begin 
        WIDGET_CONTROL, IMAGETEXT, set_value=[head_text,' Outside image'] 
        wset,DRAW25_Id 
        yim=indgen(128,40)/128 
        xim=indgen(128,40)-yim*128 
        tv,xim*2 
        wset,DRAW26_Id 
     endelse 
 
  endif 
 
  ;# DID THE RIGHT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press gt 1 then begin 
 
     ;# GO BACK TO NORMAL 
     WIDGET_CONTROL, IMAGETEXT, set_value=' '+$ 
     strmid(filename,rstrpos(filename,'\')+1,99) 
     wset,DRAW25_Id 
     yim=indgen(128,40)/128 
     xim=indgen(128,40)-yim*128 
     tv,xim*2 
     wset,DRAW26_Id 
     readpix=0 
 
   endif 
 
END 
 
;# MARQUEE EVENTS # 
PRO MARQUEE_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
  COMMON DRAW26_Comm, DRAW26_Id 
  COMMON MOUSEH, readpix, marquee, wplot 
 
  ;# IS THE EVENT JUST MOUSE MOTION? 
  if event.type eq 2 then begin 
 
     ;# READ POSITION OF CURSOR AT THIS EVENT 
     x=event.X 
     y=event.Y 
 
     ;# CONVERT TO IMAGE COORDINATES 
     xx=x 
     yy=dim(2)-(imsize(1)-y) 
 
     ;# PRINT OUTPUT 
     if selection(0) eq 0 then begin 
        new_pos='  '+string(format='(I4.1)',xx)+'  '+string(format='(I4.1)',yy) 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[out_text(0),new_pos] 
     endif 
     if selection(0) eq 1 then begin 
        pold=[selection(12),selection(13)] 
        x1=pold(0) 
        y1=dim(2)-(imsize(1)-pold(1)) 
        old_pos='  '+string(format='(I4.1)',x1)+'  '+string(format='(I4.1)',y1) 
        new_pos='  '+string(format='(I4.1)',xx)+'  '+string(format='(I4.1)',yy) 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[out_text(0),old_pos +'    '+new_pos] 
     endif 
  endif 
 
  ;# DID THE LEFT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press eq 1 then begin 
 
     ;# READ POSITION OF CURSOR AT THIS EVENT 
     x=event.X 
     y=event.Y 
 
     ;# CONVERT TO IMAGE COORDINATES 
     xx=x 
     yy=dim(2)-(imsize(1)-y) 
 
     ;# FILL IN NEW POINT (CHECK IF IT IS THE FIRST) 
     if (xx lt dim(1)) and (yy ge 0) then begin 
        if selection(0) eq 0 then begin 
           selection=fill_select(1.0,0.0,[x,y],[x,y],[x,y]) 
           WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
           WIDGET_CONTROL, IMAGETEXT, set_value=[' Left click to select second point',$ 
                                                  out_text(1)] 
        endif else begin 
           pold=[selection(12),selection(13)] 
           selection=fill_select(1.0,0.0,pold,[x,y],pold) 
           x1=pold(0) 
           y1=dim(2)-(imsize(1)-pold(1)) 
           old_pos='  '+string(format='(I4.1)',x1)+'  '+string(format='(I4.1)',y1) 
           new_pos='  '+string(format='(I4.1)',xx)+'  '+string(format='(I4.1)',yy) 
           WIDGET_CONTROL, IMAGETEXT, set_value=[' Right click when done',$ 
                                                   old_pos +'    '+new_pos] 
           selection(0)=2 
        endelse 
        ;# PLOT SCREEN OUTPUT 
        tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
        show_select 
     endif else begin 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[' Outside image',out_text(1)] 
     endelse 
  endif 
 
  ;# DID THE RIGHT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press gt 1 then begin 
 
     ;# GO BACK TO NORMAL 
     WIDGET_CONTROL, IMAGETEXT, set_value=' '+$ 
     strmid(filename,rstrpos(filename,'\')+1,99) 
     selection(0)=0.0 ;# RESET STATUS TO NO FIRST POINT 
     marquee=0 
 
   endif 
 
END 
 
 
;# WPLOT EVENTS # 
PRO WPLOT_Event, Event 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
  COMMON DRAW26_Comm, DRAW26_Id 
  COMMON MOUSEH, readpix, marquee, wplot 
 
  ;# IS THE EVENT JUST MOUSE MOTION? 
  if event.type eq 2 then begin 
 
     ;# READ POSITION OF CURSOR AT THIS EVENT 
     x=event.X 
     y=event.Y 
 
     ;# CONVERT TO IMAGE COORDINATES 
     xx=x 
     yy=dim(2)-(imsize(1)-y) 
 
     ;# PRINT OUTPUT 
     if traceplot(0) eq 0 then begin 
        new_pos='  '+string(format='(I4.1)',xx)+'  '+string(format='(I4.1)',yy) 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[out_text(0),new_pos] 
     endif 
     if traceplot(0) eq 1 then begin 
        pold=[traceplot(1),traceplot(2)] 
        x1=pold(0) 
        y1=dim(2)-(imsize(1)-pold(1)) 
        old_pos='  '+string(format='(I4.1)',x1)+'  '+string(format='(I4.1)',y1) 
        new_pos='  '+string(format='(I4.1)',xx)+'  '+string(format='(I4.1)',yy) 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[out_text(0),old_pos +'    '+new_pos] 
     endif 
  endif 
 
  ;# DID THE LEFT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press eq 1 then begin 
 
     ;# READ POSITION OF CURSOR AT THIS EVENT 
     x=event.X 
     y=event.Y 
 
     ;# CONVERT TO IMAGE COORDINATES 
     xx=x 
     yy=dim(2)-(imsize(1)-y) 
 
     ;# FILL IN NEW POINT (CHECK IF IT IS THE FIRST) 
     if (xx lt dim(1)) and (yy ge 0) then begin 
        if traceplot(0) eq 0 then begin 
           traceplot(0) = 1 
           traceplot(1) = x 
           traceplot(2) = y 
           traceplot(3) = yy 
           traceplot(4) = x 
           traceplot(5) = y 
           traceplot(6) = yy 
           ;# GET OLD LUT AND SET PLOT TO RED 
           tvlct,r,g,b,/get 
           traceplot(7)=r(1) 
           traceplot(8)=g(1) 
           traceplot(9)=b(1) 
           tvlct,0,255,0,1 ;# set 1 to green 
           ;############# 
           plots,[traceplot(1),traceplot(4)],[traceplot(2),traceplot(5)],$ 
                 psym=1,thick=2,color=1,/device 
           WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
           WIDGET_CONTROL, IMAGETEXT, set_value=[' Left click to select second point',$ 
                                                  out_text(1)] 
        endif else begin 
           traceplot(0) = 2 
           traceplot(4) = x 
           traceplot(5) = y 
           traceplot(6) = yy 
           tv,bytscl(im,contrast(0),contrast(1))>2,0,imsize(1)-dim(2) 
           plots,[traceplot(1),traceplot(4)],[traceplot(2),traceplot(5)],$ 
                 linestyle=0,thick=2,color=1,/device 
           x1=traceplot(1) 
           y1=dim(2)-(imsize(1)-traceplot(2)) 
           old_pos='  '+string(format='(I4.1)',x1)+'  '+string(format='(I4.1)',y1) 
           new_pos='  '+string(format='(I4.1)',x)+'  '+string(format='(I4.1)',yy) 
 
           WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
           WIDGET_CONTROL, IMAGETEXT, set_value=[' Right click when done',$ 
                                                   old_pos +'    '+new_pos] 
 
        endelse 
     endif else begin 
        WIDGET_CONTROL, IMAGETEXT, get_value=out_text 
        WIDGET_CONTROL, IMAGETEXT, set_value=[' Outside image',out_text(1)] 
     endelse 
 
  endif 
 
  ;# DID THE RIGHT MOUSE BUTTON CAUSE THE EVENT? 
  if event.press gt 1 then begin 
 
     ;# GET DATA AND PLOT IF 2 POINTS ARE SELECTED 
     if traceplot(0) eq 2 then begin 
        tvlct,traceplot(7),traceplot(8),traceplot(9),1 ;# RESET LUT 
        tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
        plots,[traceplot(1),traceplot(4)],[traceplot(2),traceplot(5)],$ 
              linestyle=0,thick=2,/device 
        ;if !D.WINDOW ge 1 then wdelete,1 
        ;old_window=!D.WINDOW 
        data=bbsotrac(im,[traceplot(1),traceplot(3)],[traceplot(4),traceplot(6)]) 
        ;window,1,xsize=512,ysize=512,title='  Trace Plot' 
        ;plot,data 
        ;wset,old_window 
        xarr=lindgen(n_elements(data)) 
        limits=[min(xarr),max(xarr),min(data),max(data)] 
        caption=['TRACE PLOT','Pixels','Pixel Value'] 
        ierr=execute('bbsoplot,xarr,data,limits,caption,GROUP=event.top') 
 
     endif 
 
     ;# GO BACK TO NORMAL 
     WIDGET_CONTROL, IMAGETEXT, set_value=' '+$ 
     strmid(filename,rstrpos(filename,'\')+1,99) 
     tvlct,traceplot(7),traceplot(8),traceplot(9),1 ;# RESET LUT 
     tv,bytscl(im,contrast(0),contrast(1)),0,imsize(1)-dim(2) 
     plots,[traceplot(1),traceplot(4)],[traceplot(2),traceplot(5)],$ 
           linestyle=0,thick=2,/device 
     traceplot(0)=0 ;# RESET STATUS TO NO FIRST POINT 
     wplot=0 
 
   endif 
 
END 
 
;############################## MAIN13 EVENT - TOP EVENT HANDLER ############ 
PRO MAIN13_Event, Event 
 
  common MOUSEH, readpix, marquee, wplot 
 
  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev 
 
     if (marquee eq 0) and (wplot eq 0) then begin 
 
        CASE Ev OF 
 
        'PDMENU3': PDMENU3_Event, Event 
        'PDMENU23': PDMENU23_Event, Event 
        'PDMENU7': PDMENU7_Event, Event 
        'PDMENU8': PDMENU8_Event, Event 
        'PDMENU9': PDMENU9_Event, Event 
        'DRAW25': BEGIN 
            END 
        'DRAW26': BEGIN 
            if readpix eq 0 then begin 
               ;# NO ACTION 
            endif else begin 
               READPIX_Event, Event 
            endelse 
         END 
 
         ENDCASE 
 
     endif else begin 
 
        ;# MARQUEE SELECTION 
        if marquee eq 1 then begin 
           if Ev eq 'DRAW26' then begin 
              MARQUEE_Event, Event 
           endif 
        endif 
 
        ;# WPLOT TRACE PLOT 
        if wplot eq 1 then begin 
           if Ev eq 'DRAW26' then begin 
              WPLOT_Event, Event 
           endif 
        endif 
 
     endelse 
 
END 
 
 
; DO NOT REMOVE THIS COMMENT: END MAIN13 
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST. 
 
 
;######################################################################## 
; MAIN 
;######################################################################## 
PRO bbsofits, GROUP=Group, helpfile_input 
 
  common IMDATA,im,imlast,header,oheader,imsize,contrast,ocontrast,dim,filename,$ 
         outname,helpfile,selection,oselection,traceplot,saveflag 
 
  common MOUSEH, readpix, marquee, wplot 
 
  ;# IMDATA (DEFAULTS) ###################################### 
  im=bytarr(256,256) 
  imlast=im 
  header=["NO HEADER- You did not load any FITS image!"] 
  oheader=header 
  imsize=[2048,2048] 
  contrast=[long(0),long(255)] 
  ocontrast=contrast 
  dim=size(im) 
  filename='' 
  outname='' 
  readpix=0 
  marquee=0 
  wplot=0 
  helpfile=helpfile_input 
  p1=[0.0,0.0+(imsize(1)-dim(2))] 
  p2=[dim(1)-1,(dim(2)-1)+(imsize(1)-dim(2))] 
  selection=fill_select(0.0,0.0,p1,p2,p1) 
  traceplot=lonarr(10) 
  saveflag=1 
  ;########################################################### 
 
  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0 
 
  junk   = { CW_PDMENU_S, flags:0, name:'' } 
 
  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $ 
      COLUMN=1, $ 
      MAP=1, $ 
      TITLE='  BBSOFITS - an Image Display Widget     (ver 3.0)', $ 
      XOFFSET=100, $ 
      YOFFSET=50, $ 
      UVALUE='MAIN13')
       
  
  a=!version & ver_no=fix(strmid(a.release,0,1)) 
  if ver_no gt 3 then begin 
      BASE2 = WIDGET_BASE(MAIN13, $ 
      ROW=1, $ 
      MAP=1, $ 
      /BASE_ALIGN_CENTER,UVALUE='BASE2')
  endif else begin
  BASE2 = WIDGET_BASE(MAIN13, $ 
      ROW=1, $ 
      MAP=1, $ 
      UVALUE='BASE2')
  endelse                      
 
  MenuDesc658 = [ $ 
      { CW_PDMENU_S,       3, 'File' }, $ ; 
        { CW_PDMENU_S,       0, 'Open' }, $ ; 
        { CW_PDMENU_S,       0, 'Save' }, $ ; 
        { CW_PDMENU_S,       0, 'Save as' }, $ ; 
        { CW_PDMENU_S,       0, 'Print' }, $ ; 
        { CW_PDMENU_S,       0, 'to gif' }, $ ; 
        { CW_PDMENU_S,       0, 'to jpeg' }, $ ; 
        { CW_PDMENU_S,       0, 'Restore' }, $ ; 
        { CW_PDMENU_S,       0, 'to IDL' }, $ ; 
        { CW_PDMENU_S,       2, 'Quit' } $  ; 
 
  ] 
 
 
  PDMENU3 = CW_PDMENU( BASE2, MenuDesc658, /RETURN_FULL_NAME, $ 
      UVALUE='PDMENU3') 
 
  MenuDesc3867 = [ $ 
      { CW_PDMENU_S,       3, 'Edit' }, $ ; 
        { CW_PDMENU_S,       1, 'Correct' }, $ ; 
          {CW_PDMENU_S,        0,'Dark' }, $ ; 
          {CW_PDMENU_S,        0,'Flat' }, $ ; 
          {CW_PDMENU_S,        2,'Subtract' }, $ ; 
        { CW_PDMENU_S,       1, 'Display' }, $ ; 
          { CW_PDMENU_S,       1, 'Contrast' }, $ ; 
            { CW_PDMENU_S,       0,  'Bytescl' }, $ ; 
            { CW_PDMENU_S,       2,  'LUT' }, $ ; 
          { CW_PDMENU_S,       1, 'Color Table' }, $ ; 
            { CW_PDMENU_S,       0,  'BW ramp' }, $ ; 
            { CW_PDMENU_S,       0,  'Blue ramp' }, $ ; 
            { CW_PDMENU_S,       0,  'Orange ramp' }, $ ; 
            { CW_PDMENU_S,       0,  'Red ramp' }, $ ; 
            { CW_PDMENU_S,       0,  'Green ramp' }, $ ; 
            { CW_PDMENU_S,       2,  'Other' }, $ ; 
          { CW_PDMENU_S,       0, 'Negative' }, $ ; 
          { CW_PDMENU_S,       0, 'Log' }, $ ; 
          { CW_PDMENU_S,       0, 'Log Power' }, $ ; 
          { CW_PDMENU_S,       2, 'Erase' }, $ ; 
        { CW_PDMENU_S,       1, 'Filter' }, $ ; 
          { CW_PDMENU_S,       0, 'High Pass 9x9' }, $ ; 
          { CW_PDMENU_S,       0, 'Smooth 9x9' }, $ ; 
          { CW_PDMENU_S,       0, 'Sharpen 9x9' }, $ ; 
          { CW_PDMENU_S,       0, 'Edge' }, $ ; 
          { CW_PDMENU_S,       0, 'Laplace' }, $ ; 
          { CW_PDMENU_S,       0, 'Relief' }, $ ; 
          { CW_PDMENU_S,       0, 'Median 3x3' }, $ ; 
          { CW_PDMENU_S,       2, 'Custom' }, $  ; 
        { CW_PDMENU_S,       1, 'Resize' }, $ ; 
          { CW_PDMENU_S,       0, '2 X' }, $ ; 
          { CW_PDMENU_S,       0, '0.5 X' }, $ ; 
          { CW_PDMENU_S,       0, '0.25 X' }, $ ; 
          { CW_PDMENU_S,       2, 'BBSO VMG' }, $ ; 
        { CW_PDMENU_S,       1, 'Rotate' }, $ ; 
          { CW_PDMENU_S,       0, '-90' }, $ ; 
          { CW_PDMENU_S,       0, '180' }, $ ; 
          { CW_PDMENU_S,       0, '+90' }, $ ; 
          { CW_PDMENU_S,       0, 'hflip' }, $ ; 
          { CW_PDMENU_S,       0, 'hflip and -90' }, $ ; 
          { CW_PDMENU_S,       0, 'vflip' }, $ ; 
          { CW_PDMENU_S,       0, 'hflip and +90' }, $ ; 
          { CW_PDMENU_S,       2, 'Arbitrary' }, $ ; 
        { CW_PDMENU_S,       0, 'Rebin NxN' },$ ; 
        { CW_PDMENU_S,       0, 'Annotate' }, $ ; 
        { CW_PDMENU_S,       0, 'Byte Swap' }, $ ; 
        { CW_PDMENU_S,       0, 'Refresh' }, $ ; 
        { CW_PDMENU_S,       2, 'Undo' } $ ; 
 
  ] 
 
 
  PDMENU23 = CW_PDMENU( BASE2, MenuDesc3867, /RETURN_FULL_NAME, $ 
      UVALUE='PDMENU23') 
 
  MenuDesc1002 = [ $ 
      { CW_PDMENU_S,       3, 'Info' }, $
        { CW_PDMENU_S,       0, 'Header' }, $ 
        { CW_PDMENU_S,       0, 'Histogram' }, $ 
        { CW_PDMENU_S,       0, 'Plot' }, $ 
        { CW_PDMENU_S,       2, 'Pixel Value' } $ 
 
  ] 
 
 
  PDMENU7 = CW_PDMENU( BASE2, MenuDesc1002, /RETURN_FULL_NAME, $ 
      UVALUE='PDMENU7') 
 
  MenuDesc1003 = [ $ 
      { CW_PDMENU_S,       3, 'Select' }, $ 
        { CW_PDMENU_S,       0, 'Marquee' }, $ 
        { CW_PDMENU_S,       0, 'Invert' }, $ 
        { CW_PDMENU_S,       0, 'Crop' }, $ 
        { CW_PDMENU_S,       0, 'Hide' }, $ 
        { CW_PDMENU_S,       0, 'Show' }, $ 
        { CW_PDMENU_S,       0, 'Deselect' }, $ 
        { CW_PDMENU_S,       0, 'Save' }, $ 
        { CW_PDMENU_S,       2, 'Restore' } $ 
 
 ] 
 
  PDMENU8 = CW_PDMENU( BASE2, MenuDesc1003, /RETURN_FULL_NAME, $ 
      UVALUE='PDMENU8') 
 
  MenuDesc1004 = [ $ 
      { CW_PDMENU_S,       3, 'Help' }, $ 
        { CW_PDMENU_S,       0, 'Help' }, $ 
        { CW_PDMENU_S,       2, 'About' } $ 
 
 ] 
 
  PDMENU9 = CW_PDMENU( BASE2, MenuDesc1004, /RETURN_FULL_NAME, $ 
      UVALUE='PDMENU9') 
 
  DRAW25 = WIDGET_DRAW( BASE2, $ 
      RETAIN=2, $ 
      UVALUE='DRAW25', $ 
      XSIZE=128, $ 
      YSIZE=40) 
 
  COMMON IMAGETEXT_Comm, IMAGETEXT 
  IMAGETEXT = WIDGET_TEXT( BASE2, XSIZE=35, ysize=2, UVALUE='INFOTEXT') 
 
  BASE24 = WIDGET_BASE(MAIN13, $ 
      ROW=1, $ 
      MAP=1, $ 
      TITLE='Image', $ 
      UVALUE='BASE24') 
 
  DRAW26 = WIDGET_DRAW( BASE24, $ 
      RETAIN=2, $ 
      UVALUE='DRAW26', $ 
      XSIZE=imsize(0), $ 
      X_SCROLL_SIZE=640, $ 
      YSIZE=imsize(1), $ 
      Y_SCROLL_SIZE=512, $ 
      BUTTON_EVENT=1, $ 
      MOTION_EVENT=1 ) 
 
 
  WIDGET_CONTROL, MAIN13, /REALIZE 
 
 
  ; Get drawable window index 
 
  COMMON DRAW26_Comm, DRAW26_Id 
  WIDGET_CONTROL, DRAW26, GET_VALUE=DRAW26_Id 
 
  COMMON DRAW25_Comm, DRAW25_Id 
  WIDGET_CONTROL, DRAW25, GET_VALUE=DRAW25_Id 
 
  wset,DRAW25_Id 
  y=indgen(128,40)/128 
  x=indgen(128,40)-y*128 
  tv,x*2 
  wset,DRAW26_Id 
 
  XMANAGER, 'MAIN13', MAIN13 
END 
