function bbsomedi,im,kernel,selection,imsize
;###################################################
;     BOXCAR MEDIAN WITH PRESERVED BORDERS
;################################## AJO 1996-08-14 #

;+
; NAME:
;        BBSOMEDI
;
; PURPOSE:
;        Convolution with gaussian
;
; CALLING SEQUENCE:
;        Result = BBSOMEDI(im,kernel,selection,imsize)
;
; INPUTS:
;        im          = input image
;
;        kernel      = size of smoothing kernel
;
;        selection   = selected image area (display area coordinates)
;
;        imsize      = size of currend display area
;
; OUTPUTS:
;        Smoothed image
;
; SIDE EFFECTS:
;        None
;
; COMMON BLOCKS:
;        None.
;
; RESTRICTIONS:
;        None.
;
; PROCEDURE:
;        Standard convolution (Boxcar kernel)
;
; MODIFICATION HISTORY:
;        Written by Anders Johannesson, August 1996
;
;-

ON_ERROR,2

;# FILTER
if (kernel mod 2) eq 0 then kernel=kernel+1
kh=kernel/2
dim=size(im)
new=congrid(im,dim(1)+kh*2,dim(2)+kh*2)
new(kh,kh)=im
dimn=size(new)
for i=0,kh do begin
  new(kh,i)=im(*,0)
  new(kh,dimn(2)-1-i)=im(*,dim(2)-1)
endfor
for i=0,kh do begin
  new(i,0)=new(kh+1,*)
  new(dimn(1)-1-i,0)=new(dimn(1)-kh-2,*)
endfor
fnew=median(new,kernel)
fim=fnew(kh:kh+dim(1)-1,kh:kh+dim(2)-1)

;# WHAT IS THE CURRENT SELECTION?
x1=selection(2)
x2=selection(5)
y1=selection(7)-(imsize(1)-dim(2))
y2=selection(8)-(imsize(1)-dim(2))
xmax=dim(1)-1
ymax=dim(2)-1

;# IS A MARQUEE SELECTION ACTIVE?
if (x1 gt 0) and (y1 gt 0) and (x2 lt xmax) and (y2 lt ymax) then begin

   ;# IS THE SELECTION INVERTED?
   if selection(1) eq 0 then begin
      out=im
      out(x1,y1)=fim(x1:x2,y1:y2)
   endif else begin
      out=fim
      out(x1,y1)=im(x1:x2,y1:y2)
   endelse

endif else begin
   out=fim
endelse

return,out

end
